(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common/http'), require('apollo-link'), require('apollo-link-batch'), require('graphql/language/printer'), require('apollo-angular-link-http-common')) :
    typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common/http', 'apollo-link', 'apollo-link-batch', 'graphql/language/printer', 'apollo-angular-link-http-common'], factory) :
    (factory((global.apollo = global.apollo || {}, global.apollo.link = global.apollo.link || {}, global.apollo.link.httpBatch = {}),global.ng.core,global.ng.common.http,global.apolloLink.core,null,null,global.apollo.link.httpCommon));
}(this, (function (exports,core,http,apolloLink,apolloLinkBatch,printer,apolloAngularLinkHttpCommon) { 'use strict';

    var __extends = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    var defaults = {
        batchInterval: 10,
        batchMax: 10,
        uri: 'graphql',
        method: 'POST',
    };
    var HttpBatchLinkHandler = /** @class */ (function (_super) {
        __extends(HttpBatchLinkHandler, _super);
        function HttpBatchLinkHandler(httpClient, options) {
            var _this = _super.call(this) || this;
            _this.httpClient = httpClient;
            _this.options = options;
            _this.batchInterval = options.batchInterval || defaults.batchInterval;
            _this.batchMax = options.batchMax || defaults.batchMax;
            var batchHandler = function (operations) {
                return new apolloLink.Observable(function (observer) {
                    var body = _this.createBody(operations);
                    var headers = _this.createHeaders(operations);
                    var _a = _this.createOptions(operations), method = _a.method, uri = _a.uri, withCredentials = _a.withCredentials;
                    var req = {
                        method: method,
                        url: uri,
                        body: body,
                        options: {
                            withCredentials: withCredentials,
                            headers: headers,
                        },
                    };
                    var sub = apolloAngularLinkHttpCommon.fetch(req, _this.httpClient).subscribe({
                        next: function (result) { return observer.next(result.body); },
                        error: function (err) { return observer.error(err); },
                        complete: function () { return observer.complete(); },
                    });
                    return function () {
                        if (!sub.closed) {
                            sub.unsubscribe();
                        }
                    };
                });
            };
            var batchKey = options.batchKey ||
                (function (operation) {
                    return _this.createBatchKey(operation);
                });
            _this.batcher = new apolloLinkBatch.BatchLink({
                batchInterval: _this.batchInterval,
                batchMax: _this.batchMax,
                batchKey: batchKey,
                batchHandler: batchHandler,
            });
            return _this;
        }
        HttpBatchLinkHandler.prototype.createOptions = function (operations) {
            var context = operations[0].getContext();
            return {
                method: apolloAngularLinkHttpCommon.prioritize(context.method, this.options.method, defaults.method),
                uri: apolloAngularLinkHttpCommon.prioritize(context.uri, this.options.uri, defaults.uri),
                withCredentials: apolloAngularLinkHttpCommon.prioritize(context.withCredentials, this.options.withCredentials),
            };
        };
        HttpBatchLinkHandler.prototype.createBody = function (operations) {
            var _this = this;
            return operations.map(function (operation) {
                var includeExtensions = apolloAngularLinkHttpCommon.prioritize(operation.getContext().includeExtensions, _this.options.includeExtensions, false);
                var includeQuery = apolloAngularLinkHttpCommon.prioritize(operation.getContext().includeQuery, _this.options.includeQuery, true);
                var body = {
                    operationName: operation.operationName,
                    variables: operation.variables,
                };
                if (includeExtensions) {
                    body.extensions = operation.extensions;
                }
                if (includeQuery) {
                    body.query = printer.print(operation.query);
                }
                return body;
            });
        };
        HttpBatchLinkHandler.prototype.createHeaders = function (operations) {
            return operations.reduce(function (headers, operation) {
                return apolloAngularLinkHttpCommon.mergeHeaders(headers, operation.getContext().headers);
            }, this.options.headers);
        };
        HttpBatchLinkHandler.prototype.createBatchKey = function (operation) {
            var context = operation.getContext();
            if (context.skipBatching) {
                return Math.random()
                    .toString(36)
                    .substr(2, 9);
            }
            var headers = context.headers &&
                context.headers.keys().map(function (k) { return context.headers.get(k); });
            var opts = JSON.stringify({
                includeQuery: context.includeQuery,
                includeExtensions: context.includeExtensions,
                headers: headers,
            });
            return apolloAngularLinkHttpCommon.prioritize(context.uri, this.options.uri) + opts;
        };
        HttpBatchLinkHandler.prototype.request = function (op) {
            return this.batcher.request(op);
        };
        return HttpBatchLinkHandler;
    }(apolloLink.ApolloLink));
    var HttpBatchLink = /** @class */ (function () {
        function HttpBatchLink(httpClient) {
            this.httpClient = httpClient;
        }
        HttpBatchLink.prototype.create = function (options) {
            return new HttpBatchLinkHandler(this.httpClient, options);
        };
        HttpBatchLink.decorators = [
            { type: core.Injectable },
        ];
        /** @nocollapse */
        HttpBatchLink.ctorParameters = function () { return [
            { type: http.HttpClient, },
        ]; };
        return HttpBatchLink;
    }());

    var PROVIDERS = [HttpBatchLink];
    var HttpBatchLinkModule = /** @class */ (function () {
        function HttpBatchLinkModule() {
        }
        HttpBatchLinkModule.decorators = [
            { type: core.NgModule, args: [{
                        providers: PROVIDERS,
                    },] },
        ];
        return HttpBatchLinkModule;
    }());

    exports.HttpBatchLink = HttpBatchLink;
    exports.HttpBatchLinkHandler = HttpBatchLinkHandler;
    exports.HttpBatchLinkModule = HttpBatchLinkModule;
    exports.PROVIDERS = PROVIDERS;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=bundle.umd.js.map
